# see https://cloud.google.com/functions/docs/tutorials/terraform-pubsub

resource "random_id" "main" {
  byte_length = 2
  prefix      = "search-integration-${local.environment_name}-"
}

resource "google_storage_bucket_object" "function_source" {
  name   = "${random_id.main.hex}/search_integration.zip"
  bucket = data.terraform_remote_state.holi_infra_state.outputs.gcf_sources_upload_bucket_name
  source = "../../search_integration.zip"
}

resource "google_cloudfunctions2_function" "holi-search-integration" {
  project      = data.terraform_remote_state.holi_infra_state.outputs.shared_project_id
  name         = random_id.main.hex
  location     = local.default_region
  kms_key_name = data.terraform_remote_state.holi_infra_state.outputs.artifact_registry_cloudfunctions_key_id

  build_config {
    runtime           = "python312"
    entry_point       = "process_message"
    docker_repository = data.terraform_remote_state.holi_infra_state.outputs.cloudfunctions_repository_id

    source {
      storage_source {
        bucket = data.terraform_remote_state.holi_infra_state.outputs.gcf_sources_upload_bucket_name
        object = google_storage_bucket_object.function_source.name
      }
    }
  }

  service_config {
    max_instance_count               = 1
    min_instance_count               = local.environment_name == "production" ? 1 : 0
    available_memory                 = "200Mi"
    timeout_seconds                  = 120
    available_cpu                    = 1
    max_instance_request_concurrency = 5

    environment_variables = {
      ENVIRONMENT               = local.environment_name
      TYPESENSE_HOST            = "t9vemwkibxajcorqp-1.a1.typesense.net"
      TYPESENSE_PORT            = "443"
      TYPESENSE_PROTOCOL        = "https"
      TYPESENSE_COLLECTION_NAME = local.environment_name == "production" ? "production_all" : "staging_all"
    }
    vpc_connector                  = data.terraform_remote_state.holi_infra_state.outputs.vpc_access_connector_name
    vpc_connector_egress_settings  = "PRIVATE_RANGES_ONLY"
    ingress_settings               = "ALLOW_INTERNAL_ONLY"
    all_traffic_on_latest_revision = true
    service_account_email          = data.terraform_remote_state.holi_search_integration_common_state.outputs.function_service_account

    secret_environment_variables {
      key        = "TYPESENSE_ADMIN_API_KEY"
      project_id = data.terraform_remote_state.holi_infra_state.outputs.shared_project_id
      secret     = local.environment_name == "production" ? "TYPESENSE_CLOUD_ADMIN_API_KEY_PRODUCTION" : "TYPESENSE_CLOUD_ADMIN_API_KEY_STAGING"
      version    = "latest"
    }
  }

  labels = {
    "environment_type" = local.environment_type
    "holi_service"     = "search-integration"
  }

  # taken from https://github.com/hashicorp/terraform-provider-google/issues/1938#issuecomment-1229042663
  lifecycle {
    replace_triggered_by = [
      google_storage_bucket_object.function_source
    ]
  }
}
